@@ -19,7 +19,7 @@ pub(crate) fn sys_uname(request: UNotifyEventRequest) -> ScmpNotifResp {
         let req = request.scmpreq;
 
         // SAFETY: Check pointer against mmap_min_addr.
-        let ptr = if !is_valid_ptr(req.data.args[0]) {
+        let ptr = if !is_valid_ptr(req.data.args[0], req.data.arch) {
             return Err(Errno::EFAULT);
         } else {
             req.data.args[0]
