@@ -13,7 +13,7 @@ use std::{mem::take, os::fd::AsFd};
 use memchr::{memchr, memmem};
 use nix::{errno::Errno, unistd::read};
 
-use crate::{fs::write_all, retry::retry_on_eintr};
+use crate::{io::write_all, retry::retry_on_eintr};
 
 //
 // Data types
