@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/pool.rs: Self growing / shrinking `ThreadPool` implementation
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon rusty_pool which is:
 //     Copyright (c) Robin Friedli <robinfriedli@icloud.com>
 //     SPDX-License-Identifier: Apache-2.0
@@ -29,6 +29,7 @@ use std::{
     thread,
 };
 
+use dur::Duration;
 use libseccomp::ScmpFilterContext;
 use nix::{
     errno::Errno,
@@ -42,19 +43,23 @@ use crate::{
     alert,
     config::*,
     confine::ExportMode,
+    debug,
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
-    fs::{block_signal, closeexcept, retry_on_eintr, retry_on_intr, seccomp_export_pfc},
-    hash::SydHashMap,
+    fd::closeexcept,
+    fs::{block_signal, seccomp_export_pfc},
     hook::HandlerMap,
-    info, rwrite, rwriteln,
+    info,
+    retry::{retry_on_eintr, retry_on_intr},
+    rwrite, rwriteln,
     sandbox::{Capability, Flags, Sandbox},
     workers::{
         aes::{AesLock, AesWorker},
         emu::Worker,
         int::Interrupter,
         ipc::IpcWorker,
-        UnixMap, WorkerCache, WorkerData,
+        out::Timeouter,
+        WorkerCache, WorkerData,
     },
 };
 
@@ -68,11 +73,9 @@ pub(crate) struct ThreadPool {
     keep_alive: u16,
     flags: Flags,
     seccomp_fd: RawFd,
-    pub(crate) cache: Arc<WorkerCache<'static>>,
+    pub(crate) cache: Arc<WorkerCache>,
     sandbox: Arc<RwLock<Sandbox>>,
     handlers: Arc<HandlerMap>,
-    crypt_map: Option<AesLock>,
-    unix_map: UnixMap,
     should_exit: Arc<AtomicBool>,
     worker_data: Arc<WorkerData>,
 }
@@ -100,16 +103,14 @@ impl ThreadPool {
     ) -> Self {
         Self {
             sandbox,
-            crypt_map,
             handlers,
             core_size,
             keep_alive,
             flags,
             should_exit,
             seccomp_fd,
-            cache: Arc::new(WorkerCache::new()),
+            cache: Arc::new(WorkerCache::new(crypt_map)),
             worker_data: Arc::new(WorkerData::default()),
-            unix_map: Arc::new(RwLock::new(SydHashMap::default())),
         }
     }
 
@@ -122,7 +123,7 @@ impl ThreadPool {
         match ExportMode::from_env() {
             Some(ExportMode::BerkeleyPacketFilter) => {
                 // Worker rules
-                let is_crypt = self.crypt_map.is_some();
+                let is_crypt = self.cache.crypt_map.is_some();
                 let ctx = Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, &[], &[])?;
                 let file = OpenOptions::new()
                     .write(true)
@@ -177,7 +178,7 @@ impl ThreadPool {
                     "# Syd monitor rules with seccomp fd {}",
                     self.seccomp_fd
                 )?;
-                let is_crypt = self.crypt_map.is_some();
+                let is_crypt = self.cache.crypt_map.is_some();
                 let ctx = Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, &[], &[])?;
                 rwrite!(stdout, "{}", seccomp_export_pfc(&ctx)?)?;
 
@@ -238,21 +239,7 @@ impl ThreadPool {
             .name("syd_mon".to_string())
             .stack_size(MON_STACK_SIZE)
             .spawn(move || {
-                // Get information from the sandbox about:
-                // 1. Encryption if Crypt sandboxing is enabled.
-                // 2. Transit UIDs and GIDs for SafeSetId.
-                let (fpid, crypt, transit_uids, transit_gids) = {
-                    let sandbox = self.sandbox.read().unwrap_or_else(|err| err.into_inner());
-                    let fpid = sandbox.fpid;
-
-                    let crypt = if sandbox.enabled(Capability::CAP_CRYPT) {
-                        Some((sandbox.crypt_setup()?, sandbox.crypt_tmp))
-                    } else {
-                        None
-                    };
-
-                    (fpid, crypt, sandbox.transit_uids.clone(), sandbox.transit_gids.clone())
-                };
+                let sandbox = self.sandbox.read().unwrap_or_else(|err| err.into_inner());
 
                 // SAFETY: We use exit_group(2) here to bail,
                 // because this unsharing is a critical safety feature.
@@ -270,10 +257,16 @@ impl ThreadPool {
                     ROOT_FD() as libc::c_uint,
                     PROC_FD() as libc::c_uint,
                     NULL_FD() as libc::c_uint,
-                    fpid as libc::c_uint,
+                    sandbox.fpid as libc::c_uint,
                     self.seccomp_fd as libc::c_uint,
                     crate::log::LOG_FD.load(Ordering::Relaxed) as libc::c_uint,
                 ];
+
+                let crypt = if sandbox.enabled(Capability::CAP_CRYPT) {
+                    Some((sandbox.crypt_setup()?, sandbox.crypt_tmp))
+                } else {
+                    None
+                };
                 #[expect(clippy::cast_sign_loss)]
                 if let Some((crypt_fds, crypt_tmp)) = crypt {
                     set.push(crypt_fds.0 as libc::c_uint);
@@ -287,17 +280,28 @@ impl ThreadPool {
                 drop(set);
 
                 // Spawn the interrupt thread which will confine itself.
-                self.try_spawn_interrupt(&transit_uids, &transit_gids)?;
+                self.try_spawn_interrupt(&sandbox.transit_uids, &sandbox.transit_gids)?;
 
                 // Spawn the AES thread if encryption is on.
                 let crypt_handle = if let Some((fds, tmp)) = crypt {
-                    let map = self.crypt_map.as_ref().map(Arc::clone).ok_or(Errno::ENOKEY)?;
+                    let map = self.cache.crypt_map.as_ref().map(Arc::clone).ok_or(Errno::ENOKEY)?;
                     let should_exit = Arc::clone(&self.should_exit);
-                    Some(self.try_spawn_aes(fds, map, tmp.is_none(), should_exit, &transit_uids, &transit_gids)?)
+                    Some(self.try_spawn_aes(
+                            fds,
+                            map,
+                            tmp.is_none(),
+                            should_exit,
+                            &sandbox.transit_uids,
+                            &sandbox.transit_gids)?)
                 } else {
                     None
                 };
 
+                if let Some(tmout) = sandbox.tmout {
+                    // Spawn the timeouter thread which will confine itself.
+                    self.try_spawn_timeout(tmout, &sandbox.transit_uids, &sandbox.transit_gids)?;
+                }
+
                 info!("ctx": "boot", "op": "start_monitor_thread",
                     "msg": format!("started monitor thread with pool size set to {} threads and keep alive set to {} seconds",
                         self.core_size,
@@ -314,10 +318,15 @@ impl ThreadPool {
                 //    precompute it and pass it to emulator threads for fast confinement.
                 let dry_run = env::var_os(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
                 let safe_setid = self.flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
-                let is_crypt = self.crypt_map.is_some();
+                let is_crypt = self.cache.crypt_map.is_some();
 
                 let mut ctx = if !dry_run {
-                    let ctx = Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, &transit_uids, &transit_gids)?;
+                    let ctx = Worker::prepare_confine(
+                        self.seccomp_fd,
+                        self.flags,
+                        is_crypt,
+                        &sandbox.transit_uids,
+                        &sandbox.transit_gids)?;
 
                     if Sandbox::locked_once() {
                         // Sandbox locked, confine right away.
@@ -361,8 +370,7 @@ impl ThreadPool {
                         "msg": "monitor thread is running unconfined in debug mode");
                     None
                 };
-                drop(transit_uids);
-                drop(transit_gids);
+                drop(sandbox); // release the read-lock.
 
                 info!("ctx": "boot", "op": "start_core_emulator_threads",
                     "msg": format!("starting {} core emulator thread{}, sandboxing started!",
@@ -376,7 +384,7 @@ impl ThreadPool {
 
                 // Wait for grace period to give the initial
                 // core emulator thread a chance to spawn itself.
-                std::thread::sleep(MON_GRACE_TIME);
+                std::thread::sleep(MON_GRACE_TIME.into());
 
                 loop {
                     // Confine and drop filter if sandbox is locked.
@@ -415,17 +423,26 @@ impl ThreadPool {
                         Ok(Some(_)) => {
                             // We have spawned a new emulator thread,
                             // wait for one cycle before reattempting.
-                            std::thread::sleep(MON_CYCLE_TIME);
+                            std::thread::sleep(MON_CYCLE_TIME.into());
                         }
                         Ok(None) => {
                             // We have idle threads, no need to spawn a new worker.
                             // Wait for grace period before reattempting.
-                            std::thread::sleep(MON_GRACE_TIME);
+                            std::thread::sleep(MON_GRACE_TIME.into());
                         }
-                        Err(_) => {
-                            // Caller try_spawn logs an alert level entry about this.
+                        Err(errno) => {
+                            alert!("ctx": "spawn",
+                                "msg": format!("spawn emulator failed: {errno}!"),
+                                "err": errno as i32,
+                                "core_size": self.core_size,
+                                "keep_alive": self.keep_alive);
+
+                            // Be defensive and signal stuck emulator threads to make
+                            // better use of available resources.
+                            self.signal_int();
+
                             // Wait for grace period before reattempting.
-                            std::thread::sleep(MON_GRACE_TIME);
+                            std::thread::sleep(MON_GRACE_TIME.into());
                         }
                     }
                 }
@@ -496,6 +513,48 @@ impl ThreadPool {
         Ok(handle)
     }
 
+    /// Spawn an timeout handler thread to unblock Syd syscall
+    /// handler threads when the respective sandbox process
+    /// receives a non-restarting signal.
+    pub(crate) fn try_spawn_timeout(
+        &self,
+        tmout: Duration,
+        transit_uids: &[(Uid, Uid)],
+        transit_gids: &[(Gid, Gid)],
+    ) -> SydResult<SydJoinHandle<()>> {
+        // Set up a notification pipe and wait for
+        // the timeout worker to start and unshare CLONE_F{ILE,}S.
+        let (pipe_rd, pipe_wr) = pipe2(OFlag::O_CLOEXEC)?;
+
+        let handle = retry_on_intr(|| {
+            Timeouter::new(
+                tmout,
+                self.flags,
+                transit_uids,
+                transit_gids,
+                Arc::clone(&self.should_exit),
+            )
+            .try_spawn((pipe_rd.as_raw_fd(), pipe_wr.as_raw_fd()))
+        })?;
+
+        // Wait for startup notification.
+        let mut buf = [0u8; 1];
+        match retry_on_eintr(|| read(&pipe_rd, &mut buf[..]))? {
+            0 => {
+                // timeout thread died before unshare.
+                // This should ideally never happen.
+                return Err(Errno::EIO.into());
+            }
+            1 if buf[0] == 42 => {
+                // timeout thread unshared successfully.
+                // We can go ahead and close our copies now.
+            }
+            _ => unreachable!("BUG: The meaning of life is not {:#x}!", buf[0]),
+        }
+
+        Ok(handle)
+    }
+
     /// Try to create a new encryption thread.
     pub(crate) fn try_spawn_aes(
         &self,
@@ -530,7 +589,7 @@ impl ThreadPool {
     pub(crate) fn try_spawn(
         &self,
         ctx: Option<&ScmpFilterContext>,
-    ) -> SydResult<Option<(SydJoinHandle<()>, bool)>> {
+    ) -> Result<Option<(SydJoinHandle<()>, bool)>, Errno> {
         // Create a new worker if there are no idle threads and the
         // current worker count is lower than the max pool size.
         let worker_count_val = self.worker_data.0.load(Ordering::Relaxed);
@@ -539,14 +598,16 @@ impl ThreadPool {
         let keep_alive = if curr_worker_count < self.core_size {
             // Create a new core worker if current pool size is below
             // core size during the invocation of this function.
-            crate::debug!("ctx": "spawn", "dec": "create_new_core_emulator",
+            debug!("ctx": "spawn",
+                "msg": "creating new core emulator",
                 "busy_worker_count": busy_worker_count,
                 "curr_worker_count": curr_worker_count,
                 "core_size": self.core_size);
             None
         } else if busy_worker_count < curr_worker_count {
             // We have idle threads, no need to spawn a new worker.
-            crate::debug!("ctx": "spawn", "dec": "idle_emulator_exists",
+            debug!("ctx": "spawn",
+                "msg": "idle emulator exists, no need to spawn",
                 "busy_worker_count": busy_worker_count,
                 "curr_worker_count": curr_worker_count,
                 "core_size": self.core_size);
@@ -556,7 +617,8 @@ impl ThreadPool {
             // below the EMU_MAX_SIZE and the pool has been observed to
             // be busy (no idle workers) during the invocation of this
             // function.
-            crate::debug!("ctx": "spawn", "dec": "create_new_idle_emulator",
+            debug!("ctx": "spawn",
+                "msg": "creating new idle emulator",
                 "busy_worker_count": busy_worker_count,
                 "curr_worker_count": curr_worker_count,
                 "core_size": self.core_size,
@@ -565,12 +627,13 @@ impl ThreadPool {
         } else {
             // We cannot spawn anymore workers!
             // Ideally, this should never happen.
-            crate::alert!("ctx": "spawn", "dec": "emulator_capacity_exceeded",
+            alert!("ctx": "spawn",
+                "msg": "emulator capacity exceeded, can not spawn new",
                 "busy_worker_count": busy_worker_count,
                 "curr_worker_count": curr_worker_count,
                 "core_size": self.core_size,
                 "keep_alive": self.keep_alive);
-            return Ok(None);
+            return Err(Errno::ERANGE);
         };
 
         // Try to spawn a new worker.
@@ -584,12 +647,38 @@ impl ThreadPool {
                     keep_alive,
                     Arc::clone(&self.should_exit),
                     Arc::clone(&self.worker_data),
-                    self.crypt_map.as_ref().map(Arc::clone),
-                    Arc::clone(&self.unix_map),
                 )
                 .try_spawn(ctx)
             })?,
             keep_alive.is_none(),
         )))
     }
+
+    fn signal_int(&self) {
+        // Unblock stuck emulator threads with manual signal.
+        let mut nsig = 0usize;
+
+        {
+            let (ref lock, ref cvar) = *self.cache.sysint_map.sys_block;
+            let mut map = lock.lock().unwrap_or_else(|err| err.into_inner());
+
+            for interrupt in map.iter_mut() {
+                if !interrupt.signal {
+                    interrupt.signal = true;
+                    nsig = nsig.saturating_add(1);
+                }
+            }
+
+            // Notify interrupt thread.
+            cvar.notify_one();
+        }
+
+        if nsig > 0 {
+            alert!("ctx": "spawn",
+                "msg": format!("signaled interrupt to unstuck {nsig} emulators"),
+                "nsig": nsig,
+                "core_size": self.core_size,
+                "keep_alive": self.keep_alive);
+        }
+    }
 }
