@@ -22,6 +22,7 @@ use syd::{
 #[cfg(all(
     not(coverage),
     not(feature = "prof"),
+    not(target_os = "android"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
@@ -159,9 +160,13 @@ fn help() {
 fn list() {
     println!("container");
     println!("core");
+    println!("cwd");
     println!("debug");
     println!("enforce");
     println!("firefox");
+    println!("fs");
+    println!("gui");
+    println!("hide");
     println!("immutable");
     println!("landlock");
     println!("lib");
@@ -294,6 +299,24 @@ fn dump(name: &str) -> SydResult<()> {
                 println!("{command}");
             }
         }
+        "cwd" | "pwd" => {
+            println!("# Syd profile: CWD");
+            println!("# Number of rules: {}", syd::config::PROFILE_CWD.len());
+            println!("# Copyright (c) 2025 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_CWD {
+                println!("{command}");
+            }
+        }
+        "hide" => {
+            println!("# Syd profile: Hide");
+            println!("# Number of rules: {}", syd::config::PROFILE_HIDE.len());
+            println!("# Copyright (c) 2025 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_HIDE {
+                println!("{command}");
+            }
+        }
         "noipv4" => {
             println!("# Syd profile: NoIpv4");
             println!("# Number of rules: {}", syd::config::PROFILE_NOIPV4.len());
@@ -405,6 +428,24 @@ fn dump(name: &str) -> SydResult<()> {
                 println!("{command}");
             }
         }
+        "fs" => {
+            println!("# Syd profile: Filesystem");
+            println!("# Number of rules: {}", syd::config::PROFILE_FS.len());
+            println!("# Copyright (c) 2025 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_FS {
+                println!("{command}");
+            }
+        }
+        "gui" => {
+            println!("# Syd profile: Graphical User Interface");
+            println!("# Number of rules: {}", syd::config::PROFILE_FS.len());
+            println!("# Copyright (c) 2025 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_GUI {
+                println!("{command}");
+            }
+        }
         "off" => {
             println!("# Syd profile: Off");
             println!("# Number of rules: {}", syd::config::PROFILE_OFF.len());
@@ -450,6 +491,18 @@ fn dump(name: &str) -> SydResult<()> {
                 println!("{command}");
             }
         }
+        "nixstore" => {
+            println!("# Syd profile: Nix store");
+            println!(
+                "# Number of rules: {}",
+                syd::config::PROFILE_NIX_STORE.len()
+            );
+            println!("# Copyright (c) 2025 Emery Hemingway <emery+syd@slow.janky.email>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_NIX_STORE {
+                println!("{command}");
+            }
+        }
         _ => return Err(Errno::EINVAL.into()),
     }
 
